@@ -1,14 +1,14 @@
 package OpenInteract::Handler::News;
 
-# $Id: News.pm,v 1.8 2002/08/25 15:51:00 lachoy Exp $
+# $Id: News.pm,v 1.14 2002/09/09 03:06:23 lachoy Exp $
 
 use strict;
+use base qw( OpenInteract::Handler::GenericDispatcher  SPOPS::Secure );
 use SPOPS::Secure qw( :level );
 use Data::Dumper  qw( Dumper );
 use SPOPS::Utility;
 
-@OpenInteract::Handler::News::ISA     = qw( OpenInteract::Handler::GenericDispatcher  SPOPS::Secure );
-$OpenInteract::Handler::News::VERSION = sprintf("%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Handler::News::VERSION = sprintf("%d.%02d", q$Revision: 1.14 $ =~ /(\d+)\.(\d+)/);
 
 $OpenInteract::Handler::News::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::News::default_method    = 'listing';
@@ -17,7 +17,9 @@ $OpenInteract::Handler::News::default_method    = 'listing';
  latest  => SEC_LEVEL_READ,
  listing => SEC_LEVEL_READ,  show   => SEC_LEVEL_READ, search => SEC_LEVEL_READ,
  edit    => SEC_LEVEL_WRITE, remove => SEC_LEVEL_WRITE,
- notify  => SEC_LEVEL_READ 
+ notify  => SEC_LEVEL_READ,
+ show_summary => SEC_LEVEL_WRITE,
+ edit_summary => SEC_LEVEL_WRITE,
 );
 
 # 12 weeks -- default expiration for news item
@@ -27,43 +29,64 @@ sub latest {
     my ( $class, $p ) = @_;
     my $R = OpenInteract::Request->instance;
     my $apr = $R->apache;
+    my $num_items = $p->{num_items} || $apr->param( 'num_items' ) || 5;
+    my $cached = $class->check_cache( $p, { num_items => $num_items } );
+    return $cached if ( $cached );
 
-    my $params = { error_msg   => $p->{error_msg},
-                   show_box    => $p->{show_box} };
-    $params->{num_items} = $p->{num_items} || $apr->param( 'num_items' ) || 5;
+    my %params = ( error_msg   => $p->{error_msg},
+                   show_box    => $p->{show_box},
+                   status_msg  => $p->{status_msg},
+                   num_items   => $num_items );
     my $where = "active_on <= ? AND active = ? ";
     my @values = ( SPOPS::Utility->now, 'yes' );
-    $params->{news_list} = $class->_process_listing({
+    $params{news_list} = $class->_process_listing({
                                         where => $where,
                                         value => \@values,
-                                        limit => $params->{num_items} });
-    return $R->template->handler( {}, $params,
-                                  { name => 'news::news_latest' } );
+                                        limit => $params{num_items} });
+    return $class->generate_content( $p,
+                                     { num_items => $num_items },
+                                     \%params,
+                                     { name => 'news::news_listing' } );
 }
 
 
-# List current news items
+# Just provide a listing of news titles, optionally filtered by
+# section
 
 sub listing {
     my ( $class, $p ) = @_;
     my $R = OpenInteract::Request->instance;
-    my $params = { error_msg   => $p->{error_msg} };
+    my $apr = $R->apache;
+    my $option  = $p->{option}  || $apr->param( 'option' );
+    my $section = $p->{section} || $apr->param( 'section' );
+    my %cache_params = ( option  => $option, section => $section );
+    my $cached = $class->check_cache( $p, \%cache_params );
+    return $cached if ( $cached );
+
+    my %params = ( error_msg   => $p->{error_msg},
+                   status_msg  => $p->{status_msg},
+                   section     => $section );
     my $where = " active_on <= ? AND active = ? ";
     my @values = ( SPOPS::Utility->now, 'yes' );
-    my $option = $R->apache->param( 'option' );
-    if ( $option eq 'older' ) {
-        $where .= ' AND ( expires_on < ? ) ';
-    }
-    else {
-        $where .= ' AND ( expires_on IS NULL OR expires_on > ? ) ';
-    }
-    push @values, SPOPS::Utility->now;
-    $params->{news_list} = $class->_process_listing({ where => $where,
-                                                      value => \@values });
 
-    $R->{page}{title} = 'News Items';
-    return $R->template->handler( {}, $params,
-                                  { name => 'news::news_list' } );
+    $where .= ( $option eq 'older' )
+                ? ' AND ( expires_on < ? ) '
+                : ' AND ( expires_on IS NULL OR expires_on > ? ) ';
+    push @values, SPOPS::Utility->now;
+
+    if ( $section ) {
+        $where .= ' AND ( section = ? )';
+        push @values, $section;
+    }
+
+    $params{news_list} = $class->_process_listing({ where => $where,
+                                                    value => \@values,
+                                                    column_group => 'listing' });
+    $params{section_list} = eval{ $R->news_section->fetch_group({
+                                             order => 'section' }) };
+    $R->{page}{title} = 'News Entries';
+    return $class->generate_content( $p, \%cache_params, \%params,
+                                     { name => 'news::news_list' } );
 }
 
 
@@ -78,10 +101,11 @@ sub _process_listing {
 
     my $order = $p->{order} || 'posted_on DESC';
     my $news_list = eval { $R->news->fetch_group({
-                                      where        => $p->{where},
-                                      value        => $p->{value},
-                                      limit        => $p->{limit},
-                                      order        => $order  }) };
+                                      where  => $p->{where},
+                                      value  => $p->{value},
+                                      limit  => $p->{limit},
+                                      column_group => $p->{column_group},
+                                      order  => $order  }) };
     if ( $@ ) {
         OpenInteract::Error->set( SPOPS::Error->get );
         $R->throw( { code => 403 } );
@@ -94,18 +118,21 @@ sub _process_listing {
 
     my %posters = ();
     foreach my $news ( @{ $news_list } ) {
-        ( $news->{tmp_content} ) = split '<!--BREAK-->', $news->{news_item};
+        if ( $news->{news_item} ) {
+            ( $news->{tmp_content} ) = split '<!--BREAK-->', $news->{news_item};
 
-        # If there is template content in the news item, process it
+            # If there is template content in the news item, process it
 
-        if ( $news->{tmp_content} =~ m|\[\%| ) {
-            $R->DEBUG && $R->scrib( 1, "Processing template directive content for news item ($news->{title})" );
-            $news->{tmp_content} = $R->template->handler( {}, {}, 
-                                                          { text => $news->{tmp_content} } );
+            if ( $news->{tmp_content} =~ m|\[\%| ) {
+                $R->DEBUG && $R->scrib( 1, "Processing template directive content ",
+                                           "for news item ($news->{title})" );
+                $news->{tmp_content} = $R->template->handler( {}, {},
+                                                              { text => $news->{tmp_content} } );
+            }
         }
 
-        # Now grab relevant user information, caching as we go (since the
-        # same people tend to post news)
+        # Now grab relevant user information, caching as we go (since
+        # the same people tend to post news)
 
         unless ( $posters{ $news->{posted_by} } ) {
             my $user = eval { $news->posted_by_user() };
@@ -114,7 +141,7 @@ sub _process_listing {
                 $poster_info->{login_name} = 'admin';
                 $poster_info->{user_id}    = undef;
             }
-            else { 
+            else {
                 $poster_info->{login_name} = $user->{login_name};
                 $poster_info->{user_id}    = $user->{user_id};
             }
@@ -125,78 +152,58 @@ sub _process_listing {
     return $news_list;
 }
 
-
-# (IS THIS USED ANYMORE?)
-
-sub search {
-    my ( $class, $p ) = @_;
-    my $R = OpenInteract::Request->instance;
-    my $search = $R->apache->param( 'search' );
-
-    my $params = { main_script => MAIN_SCRIPT,
-                   error_msg   => $p->{error_msg} };
-
-    my $where = " active_on <= NOW() AND active = ? ";
-    if ( $search ) {
-        $search =~ s/ /\', \'/goi;
-        my $in_clause = "IN ( '$search' )";
-        $where .= " AND ( title $in_clause OR news_item $in_clause ) ";
-    }
-    my @values = ( 'yes' );
-    my $option = $R->apache->param( 'option' );
-    if ( $option eq 'older' ) {
-        $where .= ' AND ( expires_on < ? ) ';
-    }
-    else {
-        $where .= ' AND ( expires_on IS NULL OR expires_on > ? ) ';
-    }
-    push @values, SPOPS::Utility->now;
-    $params->{news_list} = $class->_process_listing({ where => $where,
-                                                      value => \@values });
-    $R->{page}{title} = 'News Search';
-    $params->{search} = $search;
-    return $R->template->handler( {}, $params,
-                                  { name => 'news::news_list' } );
-}
-
-
-# Display detail of a news item -- displays it in editable form for those 
-# who have permission to edit the object.
+# Display detail of a news item -- displays it in editable form for
+# those who have permission to edit the object.
 
 sub show {
     my ( $class, $p ) = @_;
     my $R = OpenInteract::Request->instance;
-    my $params = { main_script => MAIN_SCRIPT,
-                   error_msg   => $p->{error_msg} };
-    my $do_edit = ( $R->apache->param( 'edit' ) and 
+    my $apr = $R->apache;
+    my %params = ( error_msg   => $p->{error_msg},
+                   status_msg  => $p->{status_msg}  );
+    my $do_edit = ( $R->apache->param( 'edit' ) and
                     $p->{level} >= SEC_LEVEL_WRITE );
 
-    $params->{news} = $p->{news} || 
-                      eval { $class->_create_object( { _class => $R->news, _id_field => 'news_id' } ) };
-    return $class->listing( { error_msg => $@ } ) if ( $@ );
+    $params{news} = $p->{news};
+    unless ( $params{news} ) {
+        my $news_id = $apr->param( 'news_id' );
+        if ( $news_id ) {
+            $params{news} = eval { $R->news->fetch( $news_id ) };
+            return $class->listing( { error_msg => $@ } ) if ( $@ );
+        }
+    }
 
     # If the user has less than WRITE access to module and there is no
     # news_id or news object, there's no reason to continue
 
-    unless ( $params->{news} or $do_edit ) {
-        my $error_msg = 'Sorry, the news object you requested does not exist. Returning to listing.';
+    unless ( $params{news} or $do_edit ) {
+        my $error_msg = 'Sorry, the news object you requested does not ' .
+                        'exist. Returning to listing.';
         return $class->listing({ error_msg => $error_msg });
     }
 
     # Object security defaults to WRITE: if there is no object,
     # then you're creating it and security then depends on the application
 
-    my $obj_level = ( $params->{news} )
-                      ? $params->{news}{tmp_security_level}
+    my $obj_level = ( $params{news} )
+                      ? $params{news}{tmp_security_level}
                       : SEC_LEVEL_WRITE;
 
-    $params->{news}  ||= $R->news->new;
-    my $tmpl_name = 'news_detail';
+    $params{news}  ||= $R->news->new;
+    my ( $tmpl_name );
     if ( $do_edit and $obj_level == SEC_LEVEL_WRITE ) {
+        $params{section_list} = eval { $R->news_section->fetch_group({
+                                                       order => 'section' }) };
         $tmpl_name = 'news_form';
+        $R->{page}{title} = ( $params{news}->is_saved )
+                              ? "Editing News: $params{news}->{title}"
+                              : "Create News";
     }
-    $R->{page}{title} = 'Detail of News Item';
-    return $R->template->handler( {}, $params,
+    else {
+         $tmpl_name = 'news_detail';
+         $R->{page}{title} = "News: $params{news}->{title}";
+    }
+    return $R->template->handler( {}, \%params,
                                   { name => "news::$tmpl_name" } );
 }
 
@@ -221,7 +228,8 @@ sub edit {
 
     my $obj_level = ( $news ) ? $news->{tmp_security_level} : SEC_LEVEL_WRITE;
     if ( $obj_level < SEC_LEVEL_WRITE ) {
-        my $user_msg = 'Sorry, you do not have access to modify this news object. Returning to listing.';
+        my $user_msg = 'Sorry, you do not have access to modify this ' .
+                       'news object. Returning to listing.';
         return $class->show({ news => $news, error_msg => $user_msg });
     }
 
@@ -238,10 +246,11 @@ sub edit {
 
     $news->{expires_on} = $class->date_read( 'expires_on' );
     $news->{active_on}  = $class->date_read( 'active_on' ) || SPOPS::Utility->now;
+    $news->{posted_on}  = $class->date_read( 'posted_on' ) || SPOPS::Utility->now;
 
     # substitute <p> for hard returns where needed
 
-    $news->{news_item} =~ s/(\r\n\r\n)(?!<p>)/$1<p>/g;
+    $news->{news_item} =~ s/(\r\n\r\n)(?!<(p|pre|blockquote)>)/$1<p>/g;
 
     # Set the expiration date if it wasn't set
 
@@ -251,16 +260,25 @@ sub edit {
         $news->{expires_on} = join '-', $time_info[5] + 1900, $time_info[4] + 1, $time_info[3];
     }
 
+    # If the image URL wasn't set to something real, clear it
+
+    $news->{image_url} = undef if ( $news->{image_url} =~ m|^http://\s*$| );
+
     # Set other defaults
 
-    $news->{posted_by} ||= $R->{auth}{user}{user_id};
-    $news->{posted_on} ||= SPOPS::Utility->now;
+    $news->{posted_by}   ||= $R->{auth}{user}->id;
+    $news->{image_align} ||= 'left';
+
     eval { $news->save };
     if ( $@ ) {
         OpenInteract::Error->set( SPOPS::Error->get );
         $R->throw( { code => 407 } );
         $p->{error_msg} = 'Cannot save news object! See error log for details.';
     }
+    else {
+        $p->{status_msg} = 'News object updated ok';
+        $class->clear_cache();
+    }
     $p->{news} = $news;
     return $class->show( $p );
 }
@@ -270,9 +288,14 @@ sub edit {
 sub remove {
     my ( $class, $p ) = @_;
     my $R = OpenInteract::Request->instance;
-    my $news = eval { $class->_create_object( { _class => $R->news, _id_field => 'news_id' } ) };
-    return $class->listing({ error_msg => $@ }) if ( $@ );
-    return $class->listing({ error_msg => 'Cannot locate news object for removal.' }) unless ( $news );
+    my $news = eval { $class->_create_object( { _class => $R->news,
+                                                _id_field => 'news_id' } ) };
+    if ( $@ ) {
+        return $class->listing({ error_msg => $@ });
+    }
+    unless ( $news ) {
+        return $class->listing({ error_msg => 'Cannot locate news object for removal.' }) ;
+    }
     if ( $news->{tmp_security_level} < SEC_LEVEL_WRITE ) {
         my $error_msg = 'Sorry, you do not have access to remove this news object. Returning to listing.';
         return $class->listing( { error_msg => $error_msg } );
@@ -284,6 +307,10 @@ sub remove {
         $R->throw( { code => 405 } );
         $p->{error_msg} = "Cannot remove news object! See error log.";
     }
+    else {
+        $p->{status_msg} = 'Removed news object ok';
+        $class->clear_cache();
+    }
     return $class->listing( $p );
 }
 
@@ -303,8 +330,11 @@ sub notify {
         my $news = $R->news->fetch( $nid );
         push @news_list, $news   if ( $news );
     }
-    if ( $R->news->notify({ email => $email, subject => 'News notification',
-                            object => \@news_list, type => 'news' }) ) {
+    my $rv = $R->news->notify({ email   => $email,
+                                subject => 'News notification',
+                                object  => \@news_list,
+                                type    => 'news' });
+    if ( $rv ) {
         return '<h2 align="center">Success!</h2>' .
                '<p>Notification sent properly!</p>';
     }
@@ -312,4 +342,79 @@ sub notify {
            '<p>Error sending email. Please check error logs!</p>';
 }
 
+# (IS THIS USED ANYMORE?)
+
+sub search {
+    my ( $class, $p ) = @_;
+    my $R = OpenInteract::Request->instance;
+    my $search = $R->apache->param( 'search' );
+
+    my $params = { error_msg   => $p->{error_msg} };
+
+    my $where = " active_on <= NOW() AND active = ? ";
+    if ( $search ) {
+        $search =~ s/ /\', \'/goi;
+        my $in_clause = "IN ( '$search' )";
+        $where .= " AND ( title $in_clause OR news_item $in_clause ) ";
+    }
+    my @values = ( 'yes' );
+    my $option = $R->apache->param( 'option' );
+    if ( $option eq 'older' ) {
+        $where .= ' AND ( expires_on < ? ) ';
+    }
+    else {
+        $where .= ' AND ( expires_on IS NULL OR expires_on > ? ) ';
+    }
+    push @values, SPOPS::Utility->now;
+    $params->{news_list} = $class->_process_listing({ where => $where,
+                                                      value => \@values });
+    $R->{page}{title} = 'News Search';
+    $params->{search} = $search;
+    return $R->template->handler( {}, $params,
+                                  { name => 'news::news_list' } );
+}
+
+# Fetch all the news items for display
+sub show_summary {
+    my ( $class, $p ) = @_;
+    my $R = OpenInteract::Request->instance;
+    my $section_list = $R->news_section->fetch_group({ order => 'section' });
+    my %params = ( error_msg    => $p->{error_msg},
+                   status_msg   => $p->{status_msg},
+                   section_list => $section_list );
+    $params{news_list} = $R->news->fetch_group({ order => 'posted_on DESC' });
+    $R->{page}{title} = 'Edit Many News Items Simultaneously';
+    return $R->template->handler( {}, \%params,
+                                  { name => 'news::news_list_editable' } );
+}
+
+
+sub edit_summary {
+    my ( $class, $p ) = @_;
+    my $R = OpenInteract::Request->instance;
+    my $apr = $R->apache;
+    my @do_edit = $apr->param( 'do_edit' );
+    my ( $success, $attempt );
+    my @errors = ();
+    foreach my $news_id ( @do_edit ) {
+        next unless ( $news_id );
+        $attempt++;
+        my $news = $R->news->fetch( $news_id );
+        $news->{title} = $apr->param( "title_$news_id" );
+        $news->{section} = $apr->param( "section_$news_id" );
+        $news->{active}  = $apr->param( "active_$news_id" );
+        eval { $news->save };
+        if ( $@ ) {
+            push @errors, "$@";
+        }
+        else {
+            $success++;
+        }
+    }
+    $class->clear_cache();
+    my %params = ( status_msg => "Attempted: $attempt; successful updates $success",
+                   error_msg  => join( "<br><br>\n", @errors ) );
+    return $class->listing( \%params );
+}
+
 1;
