@@ -1,12 +1,12 @@
 package OpenInteract::Handler::Theme;
 
-# $Id: Theme.pm,v 1.6 2002/04/13 17:23:41 lachoy Exp $
+# $Id: Theme.pm,v 1.7 2002/08/12 03:47:07 lachoy Exp $
 
 use strict;
 use SPOPS::Secure qw( :all );
 
 @OpenInteract::Handler::Theme::ISA     = qw( OpenInteract::Handler::GenericDispatcher SPOPS::Secure );
-$OpenInteract::Handler::Theme::VERSION = sprintf("%d.%02d", q$Revision: 1.6 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Handler::Theme::VERSION = sprintf("%d.%02d", q$Revision: 1.7 $ =~ /(\d+)\.(\d+)/);
 
 $OpenInteract::Handler::Theme::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::Theme::default_method    = 'listing';
@@ -92,15 +92,19 @@ sub edit {
     my $R = OpenInteract::Request->instance;
     my $apr = $R->apache;
     my $theme_id = $apr->param( 'theme_id' );
-    $R->DEBUG && $R->scrib( 1, "Trying to modify ID <<$theme_id>>" );
-    my $theme = eval { $R->theme->fetch( $theme_id ) };
-    if ( $@ ) {
-        OpenInteract::Error->set( SPOPS::Error->get );
-        $R->throw( { code => 404 } );
-        $p->{error_msg} .= "Cannot fetch requested theme. See error log for details.";
+    my ( $theme );
+    if ( $theme_id ) {
+        $R->DEBUG && $R->scrib( 1, "Trying to modify ID <<$theme_id>>" );
+        $theme = eval { $R->theme->fetch( $theme_id ) };
+        if ( $@ ) {
+            OpenInteract::Error->set( SPOPS::Error->get );
+            $R->throw( { code => 404 } );
+            $p->{error_msg} .= "Cannot fetch requested theme. See error log for details.";
+        }
+    }
+    else {
+        $theme = $R->theme->new;
     }
-
-    $theme ||= $R->theme->new;
     foreach my $field ( @{ $R->theme->field_list } ) {
         next if ( $field eq 'theme_id' );
         $theme->{ $field } = $apr->param( $field );
